# Copyright (c) 2025 Cloudification GmbH.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy
from unittest import mock

import ddt

from manilaclient import api_versions
from manilaclient.common import constants
from manilaclient import config
from manilaclient.tests.unit import utils
from manilaclient.tests.unit.v2 import fakes
from manilaclient.v2 import qos_types

cs = fakes.FakeClient(api_versions.APIVersion(constants.QOS_TYPE_VERSION))

CONF = config.CONF

LATEST_MICROVERSION = CONF.max_api_microversion


@ddt.ddt
class QosTypesTest(utils.TestCase):
    def setUp(self):
        super().setUp()
        microversion = api_versions.APIVersion(constants.QOS_TYPE_VERSION)
        mock_microversion = mock.Mock(api_version=microversion)
        self.manager = qos_types.QosTypeManager(api=mock_microversion)

    @ddt.data(
        {'expected_iops': '100'},
        {'peak_iops': '500', 'expected_iops': '100'},
    )
    def test_init(self, specs):
        info = {'specs': specs}

        qos_type = qos_types.QosType(qos_types.QosTypeManager, info)

        self.assertTrue(hasattr(qos_type, '_specs'))
        self.assertIsInstance(qos_type._specs, dict)

    def test_list_types(self):
        tl = cs.qos_types.list()
        cs.assert_called('GET', '/qos-types')
        for t in tl:
            t.api_version = api_versions.APIVersion(constants.QOS_TYPE_VERSION)
            self.assertIsInstance(t, qos_types.QosType)
            self.assertTrue(callable(getattr(t, 'get_keys', '')))
            self.assertEqual({'test1': 'test1'}, t.get_keys())

    @ddt.data(
        {'qos_type': 'fixed', 'max_iops': '100'},
        {'qos_type': 'adaptive', 'peak_iops': '200'},
        {'qos_type': 'fixed', 'max_iops': '100', 'expected_iops': '100'},
    )
    def test_create(self, specs):
        specs = copy.copy(specs)

        self.mock_object(
            self.manager, '_create', mock.Mock(return_value="fake")
        )

        result = self.manager.create(
            'test-qos-type-1',
            specs=specs,
        )

        if specs is None:
            specs = {}

        expected_specs = dict(specs)

        expected_body = {
            "qos_type": {
                "name": 'test-qos-type-1',
                "specs": expected_specs,
            }
        }

        self.manager._create.assert_called_once_with(
            "/qos-types", expected_body, "qos_type"
        )
        self.assertEqual("fake", result)

    def test_set_key(self):
        t = cs.qos_types.get(1)
        t.api_version = api_versions.APIVersion(constants.QOS_TYPE_VERSION)
        t.set_keys({'k': 'v'})
        cs.assert_called('POST', '/qos-types/1/specs', {'specs': {'k': 'v'}})

    def test_unset_keys(self):
        t = cs.qos_types.get(1)
        t.api_version = api_versions.APIVersion(constants.QOS_TYPE_VERSION)
        t.unset_keys(['k'])
        cs.assert_called('DELETE', '/qos-types/1/specs/k')

    def test_update(self):
        self.mock_object(
            self.manager, '_update', mock.Mock(return_value="fake")
        )
        qos_type = 1234
        body = dict(description="updated test description")
        expected_body = {
            "qos_type": dict(description="updated test description")
        }
        result = self.manager.update(qos_type, **body)
        self.manager._update.assert_called_once_with(
            f"/qos-types/{qos_type}", expected_body, "qos_type"
        )
        self.assertEqual("fake", result)

    def test_delete(self):
        cs.qos_types.delete(1)
        cs.assert_called('DELETE', '/qos-types/1')

    def test_get_keys_from_resource_data(self):
        version = api_versions.APIVersion(constants.QOS_TYPE_VERSION)
        manager = mock.Mock()
        manager.api = mock.Mock()
        manager.api.client = mock.Mock(return_value=None)
        manager.api.client.get = mock.Mock(return_value=(200, {}))

        valid_specs = {'test': 'test'}
        qos_type = qos_types.QosType(
            manager,
            {'specs': valid_specs, 'name': 'test'},
            loaded=True,
        )
        qos_type.api_version = version

        actual_result = qos_type.get_keys()

        self.assertEqual(actual_result, valid_specs)
        self.assertEqual(manager.api.client.get.call_count, 0)

    @ddt.data(
        {'prefer_resource_data': True, 'resource_specs': {}},
        {
            'prefer_resource_data': False,
            'resource_specs': {'fake': 'fake'},
        },
        {'prefer_resource_data': False, 'resource_specs': {}},
    )
    @ddt.unpack
    def test_get_keys_from_api(self, prefer_resource_data, resource_specs):
        version = api_versions.APIVersion(constants.QOS_TYPE_VERSION)
        manager = mock.Mock()
        manager.api = mock.Mock()
        manager.api.client = mock.Mock(return_value=None)

        valid_specs = {'test': 'test'}
        manager.api.client.get = mock.Mock(
            return_value=(200, {'specs': valid_specs})
        )

        info = {
            'name': 'test',
            'uuid': 'fake',
            'specs': resource_specs,
        }
        qos_type = qos_types.QosType(manager, info, loaded=True)
        qos_type.api_version = version

        actual_result = qos_type.get_keys(prefer_resource_data)

        self.assertEqual(actual_result, valid_specs)
        self.assertEqual(manager.api.client.get.call_count, 1)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

import ddt
from oslo_utils import uuidutils

import manilaclient
from manilaclient import exceptions
from manilaclient.tests.unit import utils
from manilaclient.v2 import client


@ddt.ddt
class ClientTest(utils.TestCase):
    def setUp(self):
        super(self.__class__, self).setUp()
        self.catalog = {
            'share': [
                {'region': 'TestRegion', 'publicURL': 'http://1.2.3.4'},
            ],
        }

    def test_adapter_properties(self):
        # sample of properties, there are many more
        retries = 3
        base_url = uuidutils.generate_uuid(dashed=False)

        s = client.session.Session()
        c = client.Client(
            session=s,
            api_version=manilaclient.API_MAX_VERSION,
            service_catalog_url=base_url,
            retries=retries,
            input_auth_token='token',
        )

        self.assertEqual(base_url, c.client.endpoint_url)
        self.assertEqual(retries, c.client.retries)

    def test_auth_via_token_invalid(self):
        self.assertRaises(
            exceptions.ClientException,
            client.Client,
            api_version=manilaclient.API_MAX_VERSION,
            input_auth_token="token",
        )

    def test_auth_via_token_and_session(self):
        s = client.session.Session()
        base_url = uuidutils.generate_uuid(dashed=False)

        c = client.Client(
            input_auth_token='token',
            service_catalog_url=base_url,
            session=s,
            api_version=manilaclient.API_MAX_VERSION,
        )

        self.assertIsNotNone(c.client)
        self.assertIsNone(c.keystone_client)

    def test_auth_via_token(self):
        base_url = uuidutils.generate_uuid(dashed=False)

        c = client.Client(
            input_auth_token='token',
            service_catalog_url=base_url,
            api_version=manilaclient.API_MAX_VERSION,
        )

        self.assertIsNotNone(c.client)
        self.assertIsNone(c.keystone_client)

    @mock.patch.object(client.Client, '_get_keystone_auth_and_session')
    def test_valid_region_name_v1(self, mock_get_auth):
        self.mock_object(client.httpclient, 'HTTPClient')
        self.mock_object(client.adapter, 'LegacyJsonAdapter')

        # Mock the auth and session returned by _get_keystone_auth_and_session
        mock_auth = mock.Mock()
        mock_session = mock.Mock()
        mock_get_auth.return_value = (mock_auth, mock_session)

        # Mock the adapter to return token and endpoint
        mocked_adapter = client.adapter.LegacyJsonAdapter.return_value
        mocked_adapter.session.get_token.return_value = 'fake_token'
        mocked_adapter.session.get_endpoint.return_value = 'http://1.2.3.4'
        mocked_adapter.auth = mock_auth

        c = client.Client(
            api_version=manilaclient.API_DEPRECATED_VERSION,
            service_type="share",
            region_name='TestRegion',
        )

        self.assertTrue(mock_get_auth.called)
        client.httpclient.HTTPClient.assert_called_with(
            'http://1.2.3.4',
            'fake_token',
            'python-manilaclient',
            insecure=False,
            cacert=None,
            cert=None,
            timeout=None,
            retries=None,
            http_log_debug=False,
            api_version=manilaclient.API_DEPRECATED_VERSION,
        )
        self.assertIsNotNone(c.client)

    @mock.patch.object(client.Client, '_get_keystone_auth_and_session')
    def test_nonexistent_region_name(self, mock_get_auth):
        self.mock_object(client.adapter, 'LegacyJsonAdapter')

        # Mock the auth and session returned by _get_keystone_auth_and_session
        mock_auth = mock.Mock()
        mock_session = mock.Mock()
        mock_get_auth.return_value = (mock_auth, mock_session)

        # Mock the adapter to return token but no endpoint (None)
        mocked_adapter = client.adapter.LegacyJsonAdapter.return_value
        mocked_adapter.session.get_token.return_value = 'fake_token'
        mocked_adapter.session.get_endpoint.return_value = None
        mocked_adapter.auth = mock_auth

        self.assertRaises(
            RuntimeError,
            client.Client,
            api_version=manilaclient.API_MAX_VERSION,
            region_name='FakeRegion',
        )
        self.assertTrue(mock_get_auth.called)
        mocked_adapter.session.get_endpoint.assert_called_with(
            mock_auth,
            interface='publicURL',
            service_type='sharev2',
            region_name='FakeRegion',
        )

    @mock.patch.object(client.Client, '_get_keystone_auth_and_session')
    def test_regions_with_same_name(self, mock_get_auth):
        self.mock_object(client.httpclient, 'HTTPClient')
        self.mock_object(client.adapter, 'LegacyJsonAdapter')

        # Mock the auth and session returned by _get_keystone_auth_and_session
        mock_auth = mock.Mock()
        mock_session = mock.Mock()
        mock_get_auth.return_value = (mock_auth, mock_session)

        # Mock the adapter to return token and endpoint
        mocked_adapter = client.adapter.LegacyJsonAdapter.return_value
        mocked_adapter.session.get_token.return_value = 'fake_token'
        mocked_adapter.session.get_endpoint.return_value = 'http://2.2.2.2'
        mocked_adapter.auth = mock_auth

        c = client.Client(
            api_version=manilaclient.API_MIN_VERSION,
            service_type='sharev2',
            region_name='SecondRegion',
        )

        self.assertTrue(mock_get_auth.called)
        mocked_adapter.session.get_endpoint.assert_called_with(
            mock_auth,
            interface='publicURL',
            service_type='sharev2',
            region_name='SecondRegion',
        )
        client.httpclient.HTTPClient.assert_called_with(
            'http://2.2.2.2',
            'fake_token',
            'python-manilaclient',
            insecure=False,
            cacert=None,
            cert=None,
            timeout=None,
            retries=None,
            http_log_debug=False,
            api_version=manilaclient.API_MIN_VERSION,
        )
        self.assertIsNotNone(c.client)

    def test_client_respects_region_name(self):
        mock_session = mock.Mock()
        mock_auth = mock.Mock()
        region = 'region1'
        mock_session.get_endpoint.return_value = 'http://fake-endpoint/'
        client.Client(
            session=mock_session,
            auth=mock_auth,
            service_type='sharev2',
            endpoint_type='public',
            region_name=region,
        )
        mock_session.get_endpoint.assert_called_once_with(
            mock_auth,
            service_type='sharev2',
            interface='public',
            region_name=region,
        )

    def _get_client_args(self, **kwargs):
        client_args = {
            'auth_url': 'http://identity.example.com',
            'api_version': manilaclient.API_DEPRECATED_VERSION,
            'username': 'fake_username',
            'service_type': 'sharev2',
            'region_name': 'SecondRegion',
            'input_auth_token': None,
            'session': None,
            'service_catalog_url': None,
            'user_id': 'foo_user_id',
            'user_domain_name': 'foo_user_domain_name',
            'user_domain_id': 'foo_user_domain_id',
            'project_name': 'foo_project_name',
            'project_domain_name': 'foo_project_domain_name',
            'project_domain_id': 'foo_project_domain_id',
            'endpoint_type': 'publicUrl',
            'cert': 'foo_cert',
        }
        client_args.update(kwargs)
        return client_args

    @ddt.data(
        {
            'auth_url': 'http://identity.example.com',
            'password': 'password_backward_compat',
            'endpoint_type': 'publicURL',
            'project_id': 'foo_tenant_project_id',
        },
        {
            'password': 'renamed_api_key',
            'endpoint_type': 'public',
            'tenant_id': 'foo_tenant_project_id',
        },
    )
    def test_client_init_no_session_no_auth_token(self, kwargs):
        def fake_url_for(version):
            if version == 'v3.0':
                return 'url_v3.0'
            else:
                return None

        self.mock_object(client.httpclient, 'HTTPClient')
        self.mock_object(client.identity.v3, 'Password')
        self.mock_object(client.adapter, 'LegacyJsonAdapter')
        self.mock_object(client.session.discover, 'Discover')
        self.mock_object(client.session, 'Session')
        client_args = self._get_client_args(**kwargs)
        client_args['api_version'] = manilaclient.API_MIN_VERSION
        self.auth_url = client_args['auth_url']

        client.session.discover.Discover.return_value.url_for.side_effect = (
            fake_url_for
        )

        # Mock the adapter to return token and endpoint
        mocked_adapter = client.adapter.LegacyJsonAdapter.return_value
        mocked_adapter.session.get_token.return_value = 'fake_token'
        mocked_adapter.session.get_endpoint.return_value = 'http://3.3.3.3'
        mocked_adapter.auth = client.identity.v3.Password.return_value

        client.Client(**client_args)

        client.httpclient.HTTPClient.assert_called_with(
            'http://3.3.3.3',
            'fake_token',
            'python-manilaclient',
            insecure=False,
            cacert=None,
            cert=client_args['cert'],
            timeout=None,
            retries=None,
            http_log_debug=False,
            api_version=manilaclient.API_MIN_VERSION,
        )

        # Verify identity.v3.Password was called with correct credentials
        client.identity.v3.Password.assert_called_with(
            auth_url='url_v3.0',
            username=client_args['username'],
            password=client_args.get('password'),
            user_id=client_args['user_id'],
            user_domain_name=client_args['user_domain_name'],
            user_domain_id=client_args['user_domain_id'],
            project_id=client_args.get(
                'tenant_id', client_args.get('project_id')
            ),
            project_name=client_args['project_name'],
            project_domain_name=client_args['project_domain_name'],
            project_domain_id=client_args['project_domain_id'],
        )

        # Verify LegacyJsonAdapter was created
        client.adapter.LegacyJsonAdapter.assert_called_with(
            session=mock.ANY,
            auth=mock.ANY,
            interface=client_args['endpoint_type'],
            service_type=client_args['service_type'],
            service_name=None,
            region_name=client_args['region_name'],
        )

        # Verify session.get_token() was called
        mocked_adapter.session.get_token.assert_called_with(mock.ANY)

        # Verify session.get_endpoint() was called
        mocked_adapter.session.get_endpoint.assert_called_with(
            mock.ANY,
            interface=client_args['endpoint_type'],
            service_type=client_args['service_type'],
            region_name=client_args['region_name'],
        )

    @mock.patch.object(client.session.discover, 'Discover', mock.Mock())
    @mock.patch.object(client.session, 'Session', mock.Mock())
    def test_client_init_no_session_no_auth_token_endpoint_not_found(self):
        self.mock_object(client.httpclient, 'HTTPClient')
        self.mock_object(client.identity.v3, 'Password')
        self.mock_object(client.adapter, 'LegacyJsonAdapter')
        client_args = self._get_client_args(
            auth_urli='fake_url',
            password='foo_password',
            tenant_id='foo_tenant_id',
        )
        discover = client.session.discover.Discover
        discover.return_value.url_for.return_value = None

        self.assertRaises(
            exceptions.CommandError, client.Client, **client_args
        )

        self.assertTrue(client.session.Session.called)
        self.assertTrue(client.session.discover.Discover.called)
        self.assertFalse(client.httpclient.HTTPClient.called)
        self.assertFalse(client.identity.v3.Password.called)
        self.assertFalse(client.adapter.LegacyJsonAdapter.called)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest.lib import exceptions as tempest_exc

from manilaclient.tests.functional.osc import base


@ddt.ddt
class ShareAccessAllowTestCase(base.OSCClientTestBase):
    def test_share_access_allow(self):
        share = self.create_share()
        access_rule = self.create_share_access_rule(
            share=share['name'],
            access_type='ip',
            access_to='0.0.0.0/0',
            wait=True,
        )

        self.assertEqual(access_rule['share_id'], share['id'])
        self.assertEqual(access_rule['state'], 'active')
        self.assertEqual(access_rule['access_type'], 'ip')
        self.assertEqual(access_rule['access_to'], '0.0.0.0/0')

        # default values
        self.assertEqual(access_rule['properties'], '')
        self.assertEqual(access_rule['access_level'], 'rw')

        access_rules = self.listing_result(
            'share', f'access list {share["id"]}'
        )
        self.assertIn(access_rule['id'], [item['ID'] for item in access_rules])

        # create another access rule with different params
        access_rule = self.create_share_access_rule(
            share=share['name'],
            access_type='ip',
            access_to='12.34.56.78',
            access_level='ro',
            properties='foo=bar',
        )

        self.assertEqual(access_rule['access_type'], 'ip')
        self.assertEqual(access_rule['access_to'], '12.34.56.78')
        self.assertEqual(access_rule['properties'], 'foo : bar')
        self.assertEqual(access_rule['access_level'], 'ro')

    @ddt.data(
        {'lock_visibility': True, 'lock_deletion': True, 'lock_reason': None},
        {'lock_visibility': False, 'lock_deletion': True, 'lock_reason': None},
        {
            'lock_visibility': True,
            'lock_deletion': False,
            'lock_reason': 'testing',
        },
        {
            'lock_visibility': True,
            'lock_deletion': False,
            'lock_reason': 'testing',
        },
    )
    @ddt.unpack
    def test_share_access_allow_restrict(
        self, lock_visibility, lock_deletion, lock_reason
    ):
        share = self.create_share()
        access_rule = self.create_share_access_rule(
            share=share['id'],
            access_type='ip',
            access_to='0.0.0.0/0',
            wait=True,
            lock_visibility=lock_visibility,
            lock_deletion=lock_deletion,
            lock_reason=lock_reason,
        )

        if lock_deletion:
            self.assertRaises(
                tempest_exc.CommandFailed,
                self.openstack,
                'share',
                params=f'access delete {share["id"]} {access_rule["id"]}',
            )
        self.openstack(
            'share',
            params=f'access delete {share["id"]} {access_rule["id"]} '
            f'--unrestrict --wait',
        )


@ddt.ddt
class ShareAccessDenyTestCase(base.OSCClientTestBase):
    @ddt.data(True, False)
    def test_share_access_deny(self, lock_deletion):
        share = self.create_share()
        access_rule = self.create_share_access_rule(
            share=share['name'],
            access_type='ip',
            access_to='0.0.0.0/0',
            wait=True,
            lock_deletion=lock_deletion,
        )

        access_rules = self.listing_result(
            'share', f'access list {share["id"]}'
        )
        num_access_rules = len(access_rules)

        delete_params = (
            f'access delete {share["name"]} {access_rule["id"]} --wait'
        )
        if lock_deletion:
            delete_params += ' --unrestrict'
        self.openstack('share', params=delete_params)

        access_rules = self.listing_result(
            'share', f'access list {share["id"]}'
        )
        self.assertEqual(num_access_rules - 1, len(access_rules))


@ddt.data
class ListShareAccessRulesTestCase(base.OSCClientTestBase):
    @ddt.data("2.45", "2.33", "2.21")
    def test_share_access_list(self, microversion):
        share = self.create_share()
        self.create_share_access_rule(
            share=share['name'],
            access_type='ip',
            access_to='0.0.0.0/0',
            wait=True,
        )

        output = self.openstack(
            'share',
            params=f'access list {share["id"]}',
            flags=f'--os-share-api-version {microversion}',
        )
        access_rule_list = self.parser.listing(output)
        base_list = ['ID', 'Access Type', 'Access To', 'Access Level', 'State']
        if microversion >= '2.33':
            base_list.append('Access Key')

        if microversion >= '2.45':
            base_list.extend(['Created At', 'Updated At'])

        self.assertTableStruct(access_rule_list, base_list)
        self.assertTrue(len(access_rule_list) > 0)

        self.create_share_access_rule(
            share=share['name'],
            access_type='ip',
            access_to='192.168.0.151',
            wait=True,
            properties='foo=bar',
        )

        output = self.openstack(
            'share',
            params=f'access list {share["id"]} --properties foo=bar',
            flags=f'--os-share-api-version {microversion}',
        )
        access_rule_properties = self.parser.listing(output)

        self.assertEqual(1, len(access_rule_properties))

        self.assertEqual(
            access_rule_properties['id'], access_rule_properties[0]['ID']
        )

    def test_share_access_list_with_filters(self):
        share = self.create_share()
        access_to_filter = '20.0.0.0/0'
        self.create_share_access_rule(
            share=share['name'],
            access_type='ip',
            access_to='0.0.0.0/0',
            wait=True,
        )
        self.create_share_access_rule(
            share=share['name'],
            access_type='ip',
            access_to=access_to_filter,
            wait=True,
        )

        output = self.openstack(
            'share',
            params=f'access list {share["id"]} --access-to {access_to_filter}',
            flags='--os-share-api-version 2.82',
        )
        access_rule_list = self.parser.listing(output)

        self.assertTrue(len(access_rule_list) == 1)


class ShowShareAccessRulesTestCase(base.OSCClientTestBase):
    def test_share_access_show(self):
        share = self.create_share()
        access_rule = self.create_share_access_rule(
            share=share['name'],
            access_type='ip',
            access_to='0.0.0.0/0',
            wait=True,
        )

        access_rule_show = self.dict_result(
            'share', f'access show {access_rule["id"]}'
        )
        self.assertEqual(access_rule_show['id'], access_rule['id'])
        self.assertEqual(access_rule_show['share_id'], share['id'])
        self.assertEqual(access_rule_show['access_level'], 'rw')
        self.assertEqual(access_rule_show['access_to'], '0.0.0.0/0')
        self.assertEqual(access_rule_show['access_type'], 'ip')
        self.assertEqual(access_rule_show['state'], 'active')
        self.assertEqual(access_rule_show['access_key'], 'None')
        self.assertEqual(
            access_rule_show['created_at'], access_rule['created_at']
        )
        self.assertEqual(access_rule_show['properties'], '')
        self.assertIn('updated_at', access_rule_show)


class SetShareAccessTestCase(base.OSCClientTestBase):
    def test_set_share_access_metadata(self):
        share = self.create_share()
        access_rule = self.create_share_access_rule(
            share=share['name'],
            access_type='ip',
            access_to='0.0.0.0/0',
            wait=True,
        )

        self.assertEqual(access_rule['properties'], '')

        self.openstack(
            'share',
            params=f'access set {access_rule["id"]} --property foo=bar',
        )

        access_rule = self.dict_result(
            'share', f'access show {access_rule["id"]}'
        )
        self.assertEqual(access_rule['properties'], 'foo : bar')

    def test_set_share_access_level(self):
        share = self.create_share()
        access_rule = self.create_share_access_rule(
            share=share['name'],
            access_type='ip',
            access_to='0.0.0.0/0',
            wait=True,
        )

        self.assertEqual(access_rule['access_level'], 'rw')

        self.openstack(
            'share', params=f'access set {access_rule["id"]} --access-level ro'
        )
        access_rule = self.dict_result(
            'share', f'access show {access_rule["id"]}'
        )
        self.assertEqual(access_rule['access_level'], 'ro')


class UnsetShareAccessRulesTestCase(base.OSCClientTestBase):
    def test_unset_share_access(self):
        share = self.create_share()
        access_rule = self.create_share_access_rule(
            share=share['name'],
            access_type='ip',
            access_to='192.168.0.101',
            wait=True,
            properties='foo=bar',
        )

        self.openstack(
            'share', params=f'access unset --property foo {access_rule["id"]}'
        )

        access_rule_unset = self.dict_result(
            'share', f'access show {access_rule["id"]}'
        )
        self.assertEqual(access_rule_unset['properties'], '')



# Copyright 2013 OpenStack Foundation
# Copyright 2015 Chuck Fouts
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from manilaclient import base


class Limits(base.Resource):
    """A collection of RateLimit and AbsoluteLimit objects."""

    def __repr__(self):
        return "<Limits>"

    @property
    def absolute(self):
        for name, value in list(self._info['absolute'].items()):
            yield AbsoluteLimit(name, value)

    @property
    def rate(self):
        for group in self._info['rate']:
            uri = group['uri']
            regex = group['regex']
            for rate in group['limit']:
                yield RateLimit(
                    rate['verb'],
                    uri,
                    regex,
                    rate['value'],
                    rate['remaining'],
                    rate['unit'],
                    rate['next-available'],
                )


class RateLimit:
    """Data model that represents a flattened view of a single rate limit."""

    def __init__(self, verb, uri, regex, value, remain, unit, next_available):
        self.verb = verb
        self.uri = uri
        self.regex = regex
        self.value = value
        self.remain = remain
        self.remaining = self.remain
        self.unit = unit
        self.next_available = next_available

    def __eq__(self, other):
        return (
            self.uri == other.uri
            and self.regex == other.regex
            and self.value == other.value
            and self.verb == other.verb
            and self.remain == other.remain
            and self.unit == other.unit
            and self.next_available == other.next_available
        )

    def __repr__(self):
        return f"<RateLimit: method={self.verb} uri={self.uri}>"


class AbsoluteLimit:
    """Data model that represents a single absolute limit."""

    def __init__(self, name, value):
        self.name = name
        self.value = value

    def __eq__(self, other):
        return self.value == other.value and self.name == other.name

    def __repr__(self):
        return f"<AbsoluteLimit: name={self.name}>"


class LimitsManager(base.Manager):
    """Manager object used to interact with limits resource."""

    resource_class = Limits

    def get(self):
        """Get a specific extension.

        :rtype: :class:`Limits`
        """
        return self._get("/limits", "limits")
